# -*- coding: utf-8 -*-
# Name           : Legende
# Description    : (c) Michael Wilhelm
# Date           : 25.08.2018
# Copyright      : (c) Michael Wilhelm
# Email          : mwilhelm@hs-harz.de
# Web            : http://www.miwilhelm.de
from qgis.PyQt.QtCore import QSettings, QTranslator, QCoreApplication
from qgis.PyQt.QtGui import QIcon
from qgis.PyQt.QtWidgets import QAction, QComboBox

from PyQt5.QtGui import QCursor, QStandardItem, QStandardItemModel

# Initialize Qt resources from file resources.py
from .resources import *

from .basis import * 	# new

import os.path


class MWLegende:

	def __init__(self, iface):
		# Save reference to the QGIS interface
		self.iface = iface
		self.canvas = self.iface.mapCanvas()			# new
		
		# initialize plugin directory
		self.plugin_dir = os.path.dirname(__file__)
		# initialize locale

		# Declare instance attributes
		self.actions = []
		#self.menu = self.tr(u'&MW Legendplugin')

		# Check if plugin was started the first time in current QGIS session
		# Must be set in initGui() to survive plugin reloads
		self.first_start = None


	def initGui(self):
		"""Create the menu entries and toolbar icons inside the QGIS GUI."""
		self.setButton()
		self.first_start = True

	def setButton(self):		
		self.toolbarMWLegende = self.iface.addToolBar(u'IGM Legende')
		self.toolbarMWLegende.setObjectName(u'IGM_Legende')
		

		self.cbColor = QComboBox(self.toolbarMWLegende)
		self.cbColor.setToolTip(u"Setzen der Strichstaerke eines Linienlayers oder einer Gruppe")
		
		
		#self.cbColor.setFixedWidth(60);
		self.toolbarMWLegende.addWidget(self.cbColor)		
		strichdicken = ["0.1","0.2","0.3","0.4","0.5","0.6","0.7","0.8","0.9",
            "1.0","1.2","1.3","1.4","1.5","1.6","1.7","1.8","1.9","2.0",
            "2.1","2.2","2.3","2.4","2.5","2.6","2.7","2.8","2.9","3.0" , 
            "3.1","3.2","3.3","3.4","3.5","3.6","3.7","3.8","3.9","4.0"  ]
		model = QStandardItemModel(self.cbColor)
		for strichdicke in strichdicken:
			item = QStandardItem(strichdicke)
			model.appendRow(item)
		self.cbColor.setModel(model)
		self.cbColor.setCurrentIndex(-1)
		

		iconBtn = QIcon( os.path.join( os.path.dirname(__file__), 'images\BnSetLineWidth.png' ) )
		title=u"Setzen der Strichstaerke eines Linienlayers oder einer Gruppe"
		self.LegendLineWidthAction = QAction( iconBtn, title, self.iface.mainWindow() )
		self.LegendLineWidthAction.setCheckable( False )
		self.LegendLineWidthAction.triggered.connect( self.LegendLineWidthActionClick)
		self.toolbarMWLegende.addAction( self.LegendLineWidthAction )		
		self.actions.append(self.LegendLineWidthAction)


		iconBtn = QIcon( os.path.join( os.path.dirname(__file__), 'images\BnSetMarkerSize.png' ) )
		title=u"Setzen der Markergroessen eines Punktlayers oder einer Gruppe"
		self.LegendMarkerSizeAction = QAction( iconBtn, title, self.iface.mainWindow() )
		self.LegendMarkerSizeAction.setCheckable( False )
		self.LegendMarkerSizeAction.triggered.connect( self.LegendMarkerSizeActionClick)
		#self.iface.addToolBarIcon( self.LegendMarkerSizeAction )
		self.toolbarMWLegende.addAction( self.LegendMarkerSizeAction )
		self.actions.append(self.LegendMarkerSizeAction)

				
		
		iconBtn = QIcon( os.path.join( os.path.dirname(__file__), 'images\BnSetColor.png' ) )
		self.LegendColorAction = QAction( iconBtn, u"Setzen der Farbe eines Layers oder einer Gruppe", self.iface.mainWindow() )
		title=u"Setzen der Farbe eines Layers oder einer Gruppe"
		self.LegendColorAction.setCheckable( False )
		self.LegendColorAction.triggered.connect( self.LegendColorActionClick)
		#self.iface.addToolBarIcon( self.LegendColorAction )
		self.toolbarMWLegende.addAction( self.LegendColorAction )
		self.actions.append(self.LegendColorAction)
		
		b=True
		if b:
			iconBtn = QIcon( os.path.join( os.path.dirname(__file__), 'images\BnSetRangeColor.png' ) )
			title="Setzt die Layer oder die Gruppe mit einem Legendentyp 'abgestufte Farbe'"
			self.LegendRangeColorAction = QAction( iconBtn, title, self.iface.mainWindow() )
			self.LegendRangeColorAction.setCheckable( False )
			self.LegendRangeColorAction.triggered.connect( self.LegendRangeColorActionClick)
			self.toolbarMWLegende.addAction( self.LegendRangeColorAction )
			self.actions.append(self.LegendRangeColorAction)

		b=False
		if b:
			iconBtn = QIcon( os.path.join( os.path.dirname(__file__), 'images\BnSetRangeKategorie.png' ) )
			self.LegendKategorieAction = QAction( iconBtn, u"Setzt den Layer oder die Gruppe mit einem Legendentyp 'abgestufte Farbe'", self.iface.mainWindow() )
			title=u"Setzt den Layer oder die Gruppe mit einem Legendentyp 'abgestufte Farbe'"
			self.LegendKategorieAction.setCheckable( False )
			self.LegendKategorieAction.triggered.connect( self.LegendKategorieActionClick)
			self.toolbarMWLegende.addAction( self.LegendKategorieAction )
			self.actions.append(self.LegendKategorieAction)

		

	def LegendLineWidthActionClick(self):
		str_input = self.cbColor.currentText();
		linewidth = toFloat(str_input)		# oder toFloat(dummy)
		if linewidth==None:
			QMessageBox.information(self.iface.mainWindow(),u"Hinweis", u"Fehlerhafte Strichstärke")
			return None					
		if linewidth<0.0:
			QMessageBox.information(self.iface.mainWindow(),u"Hinweis", u"Fehlerhafter Strichstärke\nsie muss >= 0 m sein")
			return None
		from .legendlinewidth import LegendLineWidth
		instance = LegendLineWidth(self.iface,linewidth)
		instance.run()	
	
	

	def LegendMarkerSizeActionClick(self):
		str_input = self.cbColor.currentText();
		markersize = toFloat(str_input)		# oder toFloat(dummy)
		if markersize==None:
			QMessageBox.information(self.iface.mainWindow(),u"Hinweis", u"Fehlerhafte Strichstärke")
			return None					
		if markersize<0.0:
			QMessageBox.information(self.iface.mainWindow(),u"Hinweis", u"Fehlerhafter Strichstärke\nsie muss >= 0 m sein")
			return None
		from .legendmarkersize import LegendMarkerSize
		instance = LegendMarkerSize(self.iface,markersize)
		instance.run()	
	
	
	

	def LegendColorActionClick(self):
		from .legendcolor import LegendColor
		instance = LegendColor(self.iface)
		instance.run()

		
	def LegendRangeColorActionClick(self):
		from .legendrangecolor import LegendRangeColor
		instance = LegendRangeColor(self.iface)
		instance.run()
		
		
		
		

	def LegendKategorieActionClick(self):
		from .legendkategorie import LegendKategorie
		instance = LegendKategorie(self.iface)
		instance.run()
		
		
		
	def unload(self):
		"""Removes the plugin menu item and icon from QGIS GUI."""
		for action in self.actions:
			self.iface.removeToolBarIcon(action)

		del self.toolbarMWLegende		
