# -*- coding: utf-8 -*-

#   QgsWkbTypes.PointGeometry    QgsWkbTypes.LineGeometry   QgsWkbTypes.PolygonGeometry

from qgis.PyQt.QtCore import QSettings, QTranslator, QCoreApplication
from qgis.PyQt.QtGui import QIcon
from qgis.PyQt.QtWidgets import QAction

from PyQt5.Qt import *   # clipboard

# from qgis.core import *   # QgsProject, QgsLayerTreeLayer, QgsLayerTreeGroup  POINT LINE Polygon
#import qgis.core as QGis
from qgis.core import * # QgsProject, QgsMapLayer, QgsWkbTypes, QgsCoordinateTransform, QgsPoint, QgsRaster, QgsLayerTreeLayer, QgsLayerTreeGroup

if False:
	from qgis import core
	if hasattr(core, "QGis"):
		from qgis.core import QGis
	else:
		from qgis.core import Qgis as QGis

			
# Initialize Qt resources from file resources.py
from .resources import *

import os.path
from win32clipboard import *
import tempfile

import math		# round_half_up

# registry
import sys
from winreg import *



def toInt(sValue):
	try:
		value=int(sValue)
		return value
	except:
		return None


def toFloat(sValue):
	try:
		value=float(sValue.replace(',','.'))
		return value
	except:
		return None
		
		
def round_half_up(n, decimals=0):
	multiplier = 1
	for i in range(0,decimals):
		multiplier *= 10
	
	return math.floor(n*multiplier + 0.5) / multiplier
		
		
def getTempfile(filename):		
	return os.path.join(tempfile.gettempdir(), filename)

# Ergebnis ist eine Liste mit den Koordinaten
def getGeoXYSinglePolyLine(geom):
	if QgsWkbTypes.isSingleType(geom.wkbType()):
		return geom.asPolyline()
	else:
		# return geom.asMultiPolyline()[0]
		geom.convertToSingleType()
		return geom.asPolyline()

# Ergebnis ist eine Liste mit den Koordinaten
def getGeoXYSinglePolygon(geom):
	if QgsWkbTypes.isSingleType(geom.wkbType()):
		return geom.asPolygon()
	else:
		# return geom.asMultiPolyline()[0]
		geom.convertToSingleType()
		return geom.asPolygon()
		
def getVectorLayer(iface,layertyp,layername):
	if layertyp=='Point':
		return getVectorLayerPoint(iface,layername)
	elif layertyp=='LineString':
		return getVectorLayerPolyLine(iface,layername)
	elif layertyp=='Polygon':
		return getVectorLayerPolygon(iface,layername)
	else:
		QMessageBox.information(iface.mainWindow(),u"Fehler", 'Falscher Layertyp: '+layertyp)						
		return None
		
def getVectorLayerPoint(iface, layername):
	canvas = iface.mapCanvas()
	epsg = canvas.mapSettings().destinationCrs().authid()	# EPSG:4647
	return QgsVectorLayer("Point?crs="+epsg,layername,'memory') # new

def getVectorLayerPolyLine(iface, layername):
	canvas = iface.mapCanvas()
	epsg = canvas.mapSettings().destinationCrs().authid()	# EPSG:4647
	return QgsVectorLayer("LineString?crs="+epsg,layername,'memory') # new

def getVectorLayerPolygon(iface, layername):
	canvas = iface.mapCanvas()
	epsg = canvas.mapSettings().destinationCrs().authid()	# EPSG:4647
	return QgsVectorLayer("Polygon?crs="+epsg,layername,'memory') # new
	
def getVectorLayerPolyLineOGR(iface, layername):
	canvas = iface.mapCanvas()
	epsg = canvas.mapSettings().destinationCrs().authid()	# EPSG:4647
	return QgsVectorLayer("LineString?crs="+epsg,layername,'ogr') # new
	
	
# ok
def allLayerFromType(iface, geometryType, root):
	list=[]
	layers = QgsProject.instance().mapLayers().values()
	for layer in layers:
		#print('layer.name()',layer.name(),'layer.source()',layer.source(),'layer.geometryType()',layer.geometryType(), 'typ',type(layer),'geometryType',geometryType)
		if isinstance(layer,QgsVectorLayer) and layer.geometryType() == geometryType:
			#print('     ',layer.name(),'layer.geometryType()',layer.geometryType())
			#  LineString  Polygon  Point
			anzShp = layer.source().count('.shp')
			if anzShp>0 or layer.source().startswith('LineString') or layer.source().startswith('Polygon') or layer.source().startswith('Point'):
				list.append(layer)
				#print('  apppend',layer.name())
				
			else:
				#print('text-shape',layer.source())
				pass
			#print("++++++++++++++","layer",layer.name())
			
	return list		
	

	
# ok
# return the layer of the group with the name 'groupname'
def getEreignisLayerFromGroup(iface, groupname):
	root = QgsProject.instance().layerTreeRoot()
	for node in root.children():
		if isinstance(node,QgsLayerTreeGroup):
			if node.name()==groupname:
				list=[]
				for item in node.children():
					if isinstance(item,QgsLayerTreeGroup):
						pass
					else:
						layer = item.layer()
						if layer.geometryType() == QgsWkbTypes.PointGeometry or layer.geometryType() == QgsWkbTypes.LineGeometry:
							gefunden=0
							for feld in layer.fields():		# QgsFields
								if feld.name()=="ERGNR":
									gefunden+=1
								if feld.name()=="PINDEX":
									gefunden+=1
								if feld.name()=="GEWINDEX":
									gefunden+=1
								if feld.name()=="GEWNR":
									gefunden+=1
								if feld.name()=="GEWNAME":
									gefunden+=1
								if feld.name()=="VON":
									gefunden+=1
								if feld.name()=="BIS":
									gefunden+=1
							# QMessageBox.information(iface.mainWindow(),u"gefunden", str(gefunden))						
							if gefunden==7:
								list.append(layer)
				return list
	return None
	
# ok
def getLayersFromGroup(iface, groupname):
	root = QgsProject.instance().layerTreeRoot()
	for node in root.children():
		if isinstance(node,QgsLayerTreeGroup):
			if node.name()==groupname:
				list=[]
				for item in node.children():
					if isinstance(item,QgsLayerTreeGroup):
						pass  # getLayersFromExistingGroup
					else:
						# nun aktuellen Layer in der Gesamtliste suchen
						list.append(item.layer())
				return list
	return None

	
def getLayersFromGroupRekursiv(iface, groupname):
	root = QgsProject.instance().layerTreeRoot()
	for node in root.children():
		if isinstance(node,QgsLayerTreeGroup):
			if node.name()==groupname:
				list=[]
				for item in node.children():
					if isinstance(item,QgsLayerTreeGroup):
						layers = getLayersFromExistingGroup(iface,item)
						for layer in layers:
							list.append(layer)
					else:
						# nun aktuellen Layer in der Gesamtliste suchen
						list.append(item.layer())
				return list
	return None

	
	
# ok
# gibt die Liste der Einträge einer Gruppe zurück
# Liste beinhaltet Knoten, mit node.layer() bekommt man den echten Layer
def getNodeLayersFromGroup(iface, groupname):
	root = QgsProject.instance().layerTreeRoot()
	for node in root.children():
		if isinstance(node,QgsLayerTreeGroup):
			if node.name()==groupname:
				list=[]
				for item in node.children():		# QList< QgsLayerTreeNode * >
					if isinstance(item,QgsLayerTreeGroup):
						pass
					else:
						# nun aktuellen Layer in der Gesamtliste suchen
						list.append(item)
				return list
	return None

# ok
def getLayersFromGroupLower(iface, groupname):
	groupname = groupname.lower()
	root = QgsProject.instance().layerTreeRoot()
	for node in root.children():
		if isinstance(node,QgsLayerTreeGroup):
			if node.name().lower()==groupname.lower():
				list=[]
				for item in node.children():
					if isinstance(item,QgsLayerTreeGroup):
						pass
					else:
						# nun aktuellen Layer in der Gesamtliste suchen
						list.append(item.layer())
				return list
	return None
	
	# ok
	# return the layers of the group with the name 'groupname'
def getGewasserLayerFromGroup(iface, groupname):
	root = QgsProject.instance().layerTreeRoot()
	igmgew2Group = root.findGroup(groupname)
	if igmgew2Group==None:
		return None
	else:
		#  QGis.Point  QGis.Line   QGis.Polygon
		list=[]
		for item in igmgew2Group.children():
			# die children  sind QgsLayerTreeLayer !
			# mit layer() erhält man den QgsMapLayer (Oberklasse von Raster,Vector)
			if isinstance(item,QgsLayerTreeGroup):
				continue
				
			layer = item.layer()
			#if item.type()==QgsMapLayer.VectorLayer:
			if isinstance(layer,QgsVectorLayer):
				if layer.geometryType() == QgsWkbTypes.LineGeometry:
					gefunden=0
					for feld in layer.fields():		# QgsFields
						if feld.name()=="GEWINDEX":
							gefunden+=1
						if feld.name()=="IGMGEWNR":
							gefunden+=1
						if feld.name()=="IGMGEWNAME":
							gefunden+=1
						if feld.name()=="IGMLEN":
							gefunden+=1
					# QMessageBox.information(iface.mainWindow(),u"gefunden", str(gefunden))						
					if gefunden==4:
						list.append(layer)
		return list
	return None

	# ok
def getRasterLayersFromGroup(iface, groupname):
	# print("groupname: "+groupname)
	root = QgsProject.instance().layerTreeRoot()
	for node in root.children():
		if isinstance(node,QgsLayerTreeGroup):   #  extends QgsLayerTreeNode
			if node.name()==groupname:
				# print("groupname Rastername: "+node.name())
				list=[]
				# QgsRasterLayer QgsVectorLayer
				i=0
				# print("len(node.children()): "+str(len(node.children())))
				for item in node.children():  #  QgsLayerTreeNode
					i=i+1
					if item.nodeType()==1:
						layer = item.layer()
						if isinstance(layer,QgsRasterLayer):
							list.append(layer)
							# print("    gefunden  Rasterlayer.name:",layer.name())
				return list
	return None

# ok
def getLayersFromExistingGroup(iface, group):
	if group==None:
		return None
	else:
		list=[]
		for item in group.children():
			# die children  sind QgsLayerTreeLayer !
			# mit layer() erhält man den QgsMapLayer (Oberklasse von Raster,Vector)
			if isinstance(item,QgsLayerTreeGroup):
				continue
				
			layer = item.layer()
			#if item.type()==QgsMapLayer.VectorLayer:
			if isinstance(layer,QgsVectorLayer):
				list.append(layer)
		return list
	return None
	

# ok
def getPointLayerFromExistingGroup(iface, group):
	if group==None:
		return None
	else:
		list=[]
		for item in group.children():
			# die children  sind QgsLayerTreeLayer !
			# mit layer() erhält man den QgsMapLayer (Oberklasse von Raster,Vector)
			if isinstance(item,QgsLayerTreeGroup):
				continue
				
			layer = item.layer()
			#if item.type()==QgsMapLayer.VectorLayer:
			if isinstance(layer,QgsVectorLayer):
				if layer.geometryType() == QgsWkbTypes.PointGeometry:
					list.append(layer)
		return list
	return None
	
# ok
def getPolylineLayerFromExistingGroup(iface, group):
	if group==None:
		return None
	else:
		list=[]
		for item in group.children():
			# die children  sind QgsLayerTreeLayer !
			# mit layer() erhält man den QgsMapLayer (Oberklasse von Raster,Vector)
			if isinstance(item,QgsLayerTreeGroup):
				continue
				
			layer = item.layer()
			if isinstance(layer,QgsVectorLayer):
				if layer.geometryType() == QgsWkbTypes.LineGeometry:
					list.append(layer)
		return list
	return None

# ok
# return the layer of the group with the name 'groupname'
def getPolylineLayer(iface, groupname):
	root = QgsProject.instance().layerTreeRoot()
	group = root.findGroup(groupname)
	if group==None:
		return None
	else:
		#  QGis.Point  QGis.Line   QGis.Polygon
		list=[]
		for item in group.children():
			# die children  sind QgsLayerTreeLayer !
			# mit layer() erhält man den QgsMapLayer (Oberklasse von Raster,Vector)
			if isinstance(item,QgsLayerTreeGroup):
				continue
				
			layer = item.layer()
			#if item.type()==QgsMapLayer.VectorLayer:
			if isinstance(layer,QgsVectorLayer):
				if layer.geometryType() == QgsWkbTypes.LineGeometry:
					list.append(layer)
		return list
	return None

	
# ok
def getLayer(iface, layername):
	root = QgsProject.instance().layerTreeRoot()
	layers = QgsProject.instance().mapLayers().values()
	for layer in layers:
		if isinstance(layer,QgsVectorLayer):
			if layer.name()==layername:
				return layer
	return None

# ok
# return alle polgon-layer
def getPolygonLayer(iface):
	list=[]
	layers = QgsProject.instance().mapLayers().values()
	for layer in layers:
		if isinstance(layer,QgsVectorLayer):
			if layer.geometryType() == QgsWkbTypes.PolygonGeometry:
				list.append(layer)
	return list


	
# ok
def getGroup(iface, groupname):
	root = QgsProject.instance().layerTreeRoot()
	for node in root.children():
		if isinstance(node,QgsLayerTreeGroup):
			if node.name()==groupname:
				return node
	return None

def getGroupsFromGroup(group):
	groups=[]
	for node in group.children():
		if isinstance(node,QgsLayerTreeGroup):
			groups.append(node)
	return groups
	
	

# ok	
# return nur den Namen
def getGroups(iface):
	list=[]
	root = QgsProject.instance().layerTreeRoot()
	for node in root.children():
		if isinstance(node,QgsLayerTreeGroup):
			#	print(node.name())
			list.append(node.name())
	return list
		

def readRegisterInt(key, defaultvalue):
	s = QSettings()
	erg = s.value(key, defaultvalue)
	if erg==None:
		return defaultvalue
	else:
		if type(erg)=="<class 'int'>":
			return erg
		else:
			return int(erg)

def readRegisterString(key, defaultvalue):
	s = QSettings()
	erg = s.value(key, defaultvalue)
	if erg==None:
		return defaultvalue
	else:
		if type(erg)=="<class 'str'>":
			return erg
		else:
			return str(erg)
		
		
def writeRegister(key, defaultvalue):
	s = QSettings()
	erg = s.setValue(key, defaultvalue)
		
		
def readRegisterPy(regpath, searchkey):
	try:
		key = OpenKey(HKEY_CURRENT_USER, regpath, 0, KEY_ALL_ACCESS)
		res=QueryValueEx(key,searchkey)
		CloseKey(key)
		return res[0]  # res is a tuple
	except EnvironmentError:
		return None

		
def writeRegisterPyString(regpath, searchkey, value):
	try:
		key = CreateKey(HKEY_CURRENT_USER, regpath)
		key = OpenKey(HKEY_CURRENT_USER, regpath, 0, KEY_WRITE)
		res=SetValueEx(key,searchkey,0,REG_SZ, value)
		CloseKey(key)
		return True
	except EnvironmentError:
		return False
		
		
def writeRegisterPyDWord(regpath, searchkey, value):
	try:
		key = CreateKey(HKEY_CURRENT_USER, regpath)
		key = OpenKey(HKEY_CURRENT_USER, regpath, 0,  KEY_WRITE) #KEY_ALL_ACCESS)
		#   SetValueEx(aKey,"MyNewKey",0, REG_SZ, r"c:\winnt\explorer.exe") 
		res=SetValueEx(key,searchkey,0,REG_DWORD, value)
		CloseKey(key)
		return True
	except EnvironmentError:
		return False

		

# import win32clipboard
def copy2Clipboard(text):
	clipboard = QApplication.clipboard()
	clipboard.setText(text+u"\n")

		