#!/usr/bin/python
# -*- coding: utf-8 -*-


# Name           : MWLegende
# Description    : (c) Michael Wilhelm
# Date           : 25.08.2018
# Copyright      : (c) Michael Wilhelm
# Email          : mwilhelm@hs-harz.de
# Web            : http://www.miwilhelm.de
# Import the PyQt and QGIS libraries
from qgis.PyQt.QtCore import *  # QSettings, QTranslator, QCoreApplication
from qgis.PyQt.QtGui import *  # QIcon, QCursor
from qgis.PyQt.QtWidgets import *  # QAction

from qgis.gui import *

from qgis.core import * # QgsProject, QgsMapLayer, QgsWkbTypes, QgsMapLayerRegistry, QgsCoordinateTransform, QgsPoint, QgsRaster, QgsLayerTreeLayer, QgsLayerTreeGroup

# Initialize Qt resources from file resources.py
from .resources import *		# new  
from .basis import * 	# new
from .vectorlayer import * 	# new


from  .uilegendkategorie  import  uiLegendKategorie




class Farbe():
	def __init__(self, r,g,b):
		self.r=r
		self.g=g
		self.b=b;


class LegendKategorie:

	def __init__(self, iface):
		# Save reference to the QGIS interface
		self.iface = iface
		self.canvas = self.iface.mapCanvas()
		self.view = iface.layerTreeView()

	def run(self):
		layers=[]
		
		mainTree = self.iface.layerTreeView()
		currentgroup = mainTree.currentGroupNode()

		layer = getActivePolylineLayer(self.iface)
		if currentgroup==None and layer==None:
			QMessageBox.information(self.iface.mainWindow(),"Hinweis", "Es gab kein aktives Linienshape oder aktive Gruppe")
			return
		if layer==None:
			# currentgroup
			layers = getPolylineLayerFromExistingGroup(self.iface,currentgroup)
			if layers==None:
				QMessageBox.information(self.iface.mainWindow(),"Hinweis", "Die Gruppe hat keine Shapes")
				return
		else:
			layers.append(layer)
			
	
			
		hoehenliste=[]
		for layer in layers:
			#print(layer.name())
			if layer.geometryType() == QgsWkbTypes.LineGeometry:
				for feat in layer.getFeatures():
					try:
						id=feat["ID"]
						contour=feat["CONTOUR"]
						length=feat["LENGTH"]
						#k=feat["K"]
						#hoehe=feat["HL_TXT"]
						hoehenliste.append(layer)
						break
					except:
						break
						
		if len(hoehenliste)==0:
			QMessageBox.information(self.iface.mainWindow(),u"Hinweis", u"Keine Hoehenlayer trotz Gruppe vorhanden")
			return
			
			
		firstSichtbar=True
		minValueSichtbar=0
		maxValueSichtbar=0
		
		firstAlle=True
		minValueAlle=0
		maxValueAlle=0
		
		alleLayer=0
		sichtbareLayer=0
		listeAllerLayer=[]
		listeSichtbarerLayer=[]

		legendIface = self.iface.legendInterface()
		for layer in hoehenliste:
			alleLayer+=1
			listeAllerLayer.append(layer.name())
			if legendIface.isLayerVisible(layer):
				sichtbar=True
				sichtbareLayer+=1
				listeSichtbarerLayer.append(layer.name())
				#print("sichtbar",layer.name())
			else:
				sichtbar=False
				#print("nicht sichtbar",layer.name())
				
			for feat in layer.getFeatures():
				contour=feat["CONTOUR"]
				if sichtbar:
					if firstSichtbar:
						minValueSichtbar=contour
						maxValueSichtbar=contour
						firstSichtbar=False
					else:
						if contour<minValueSichtbar:
							minValueSichtbar=contour
						if contour>maxValueSichtbar:
							maxValueSichtbar=contour
						
				if firstAlle:
					minValueAlle=contour
					maxValueAlle=contour
					firstAlle=False
				else:
					if contour<minValueAlle:
						minValueAlle=contour
					if contour>maxValueAlle:
						maxValueAlle=contour
						
						
		str_minValueSichtbar=u"{0:.2f}".format(float(minValueSichtbar))
		str_maxValueSichtbar=u"{0:.2f}".format(float(maxValueSichtbar))
		str_minValueAlle=u"{0:.2f}".format(float(minValueAlle))
		str_maxValueAlle=u"{0:.2f}".format(float(maxValueAlle))
		
		# dialog Variablen
		sichtbar = False
		minValue = 0
		maxValue=0
		intervall=1
		strichdicke=0
		strichdicken = ["0.1","0.2","0.3","0.4","0.5","0.6","0.7","0.8","0.9","1.0","1.2","1.3","1.4","1.5","1.6","1.7"]
		
		anzIntervalle = ["2","3","4","5","6","7","8","9","10","11","12"]
		
		while True:
			dialog = uiLegendKategorie()
			dialog.setValues(listeAllerLayer, listeSichtbarerLayer, minValueSichtbar,maxValueSichtbar,minValueAlle,maxValueAlle)
			
			dialog.tMin.setText(str_minValueAlle)
			dialog.tMinVorgabe.setText(str_minValueAlle)
			dialog.tMax.setText(str_maxValueAlle)
			dialog.tMaxVorgabe.setText(str_maxValueAlle)
				
			
			model = QStandardItemModel(dialog.listView)
			for layer in hoehenliste:
				item = QStandardItem(layer.name())
				# item.setCheckable(True)		# Add a checkbox to it
				model.appendRow(item)			# Add the item to the model
			dialog.listView.setModel(model)		
			
			
			farbpalette = ["3D","2x gelb, orange, gruen, blau, rot, violett","3x gelb, orange, gruen, blau, rot, violett","grau"]
			model = QStandardItemModel(dialog.cbFarbpalette)
			for bez in farbpalette:
				item = QStandardItem(bez)
				model.appendRow(item)
			dialog.cbFarbpalette.setModel(model)
			dialog.cbFarbpalette.setCurrentIndex(1)
			
			
			
			model = QStandardItemModel(dialog.cbStrichdicke)
			for dicke in strichdicken:
				item = QStandardItem(dicke)
				model.appendRow(item)
			dialog.cbStrichdicke.setModel(model)
			dialog.cbStrichdicke.setCurrentIndex(3)
			
			
			
			model = QStandardItemModel(dialog.cbAnzIntervalle)
			for anzInt in anzIntervalle:
				item = QStandardItem(anzInt)
				model.appendRow(item)
			dialog.cbAnzIntervalle.setModel(model)
			dialog.cbAnzIntervalle.setCurrentIndex(5)
			
			
			dialog.chkSichtbar.setEnabled(sichtbareLayer<alleLayer)
			
			result = dialog.exec_()
			if result:			
				sichtbar = dialog.chkSichtbar.isChecked()
				minValue = self.checkInput(dialog.tMin.text(),"Minimaler Kategoriewert")
				if minValue==None:
					continue					
				maxValue = self.checkInput(dialog.tMax.text(),"Maximaler Kategoriewert")
				if maxValue==None:
					continue
				intervall = None
				if dialog.rb01.isChecked():
					intervall=0.1
				elif dialog.rb05.isChecked():
					intervall=0.5
				elif dialog.rb1.isChecked():
					intervall=1
				elif dialog.rb2.isChecked():
					intervall=2
				elif dialog.rb3.isChecked():
					intervall=3
				elif dialog.rb4.isChecked():
					intervall=4
				elif dialog.rb5.isChecked():
					intervall=5
				elif dialog.rb10.isChecked():
					intervall=10
				if intervall==None:
					QMessageBox.information(self.iface.mainWindow(),u"Hinweis", u"Der Intervallwert konnte nicht ermittelt werden")
					continue
					
				palettenIndex =  dialog.cbFarbpalette.currentIndex()
				str_strichdicke=dialog.cbStrichdicke.currentText()
				strichdicke = self.checkInputPositiv(str_strichdicke,"Strichdicke")
				if strichdicke==None:
					continue
				
				if palettenIndex==0:
					intervall = (maxValue-minValue)/9.0
				
				break
				
			else:
				return	# Abbruch

		palette=[]
		
		if palettenIndex==0:
			palette.append( Farbe(255,255,0)) # türkisgelb
			palette.append( Farbe(229,254,0))  #  hell gelb grün
			palette.append( Farbe(205,254,0))  # grün gelb
			palette.append( Farbe(116,254,0))  # hell grün
			palette.append( Farbe(2,234,0))  #  grün
			palette.append( Farbe(0,219,153))  # grün blau
			palette.append( Farbe(0,135,165))  # dunkel grün blau
			palette.append( Farbe(0,80,165))  # blau
			palette.append( Farbe(13,0,130))  # dunkel blau
			
		elif palettenIndex==1:
			palette.append( Farbe(255,255,183))  # gelb
			palette.append( Farbe(255,255,0))
				
			palette.append( Farbe(255,186,117))  # orange
			palette.append( Farbe(255,128,0))

			palette.append( Farbe(128,255,117))  # grün
			palette.append( Farbe(0,255,0))

			palette.append( Farbe(0,128,255))  # blau
			palette.append( Farbe(0,0,255))
			
			palette.append( Farbe(255,128,0))  # rot
			palette.append( Farbe(255,0,0))
			
			palette.append( Farbe(255,147,201))  # violett
			palette.append( Farbe(255,0,128))
			
		elif palettenIndex==2:
			palette.append( Farbe(255,255,183))  # gelb
			palette.append( Farbe(255,255,89))  # gelb
			palette.append( Farbe(255,255,0))
				
			palette.append( Farbe(255,186,117))  # orange
			palette.append( Farbe(255,186,45))  # orange
			palette.append( Farbe(255,128,0))

			palette.append( Farbe(128,255,117))  # grün
			palette.append( Farbe(128,255,55))  # grün
			palette.append( Farbe(0,255,0))

			palette.append( Farbe(0,128,255))  # blau
			palette.append( Farbe(0,65,255))  # blau
			palette.append( Farbe(0,0,255))
			
			palette.append( Farbe(255,128,0))  # rot
			palette.append( Farbe(255,65,0))  # rot
			palette.append( Farbe(255,0,0))
			
			palette.append( Farbe(255,147,201))  # violett
			palette.append( Farbe(255,45,201))  # violett
			palette.append( Farbe(255,0,128))
		else:
			rgb=230
			while rgb>0:
				palette.append( Farbe(rgb,rgb,rgb))  
				rgb-=15
		
		#print("sichtbar",sichtbar)
		#print("minH",minH)
		#print("maxH",maxH)
		#print("intervall",intervall)
		#print("strichdicke",strichdicke)
		
		# sichtbar = False    minH = 0   maxH=0   intervall=1
		minvalues=[]
		maxvalues=[]
		labels=[]
		hoehe = minValue
		while hoehe<=maxValue:
			#print("hoehe",hoehe)
			minvalues.append(hoehe)
			maxvalues.append(hoehe+intervall)
			str1=u"{0:.2f}".format(float(hoehe))
			str2=u"{0:.2f}".format(float(hoehe+intervall))
			labels.append(str1+"  "+str2)
			hoehe+=intervall
		
		range_list = []
		for i in range(0,len(labels)):
			symb = QgsSymbol.defaultSymbol( QgsWkbTypes.LineGeometry )
			colorindex = i % len(palette)
			color = QColor(palette[colorindex].r, palette[colorindex].g, palette[colorindex].b)
			symb.setColor( color )
			symb.setOpacity(1)
			#symb.symbolLayer(0).setBorderStyle( Qt.NoPen )	# Umrandung bei Flächen
			symb.setWidth(strichdicke) 
			label = labels[i]			# +": "+str(minvalues[i])+" -> "+str(maxvalues[i])
			range_renderer = QgsRendererRange(minvalues[i], maxvalues[i], symb, label)
			range_list.append(range_renderer)

		renderer = QgsGraduatedSymbolRenderer('', range_list)
		# renderer.setMode(QgsGraduatedSymbolRenderer.EqualInterval)
		renderer.setClassAttribute(u"CONTOUR")
		
		if sichtbar:
			for layer in hoehenliste:
				if legendIface.isLayerVisible(layer):
					layer.setRenderer(renderer)
					#self.iface.legendInterface().refreshLayerSymbology( layer )
					self.iface.layerTreeView().refreshLayerSymbology(layer.id())
					self.iface.mapCanvas().refresh()
					layer.triggerRepaint()  	# repaintRequested
		else:
			for layer in hoehenliste:
				layer.setRenderer(renderer)
				#self.iface.legendInterface().refreshLayerSymbology( layer )
				self.iface.layerTreeView().refreshLayerSymbology(layer.id())
				self.iface.mapCanvas().refresh()
				layer.triggerRepaint()  

			
	def checkInputPositiv(self,str_input,uibez):
		input = toFloat(str_input)		# oder toFloat(dummy)
		if input==None:
			QMessageBox.information(self.iface.mainWindow(),u"Hinweis", u"Fehlerhafter "+uibez)
			return None					
		if input<0.0:
			QMessageBox.information(self.iface.mainWindow(),u"Hinweis", u"Fehlerhafter "+uibez+"\nsie muss >= 0 m sein")
			return None
		return input			
		
		
	def checkInput(self,str_input,uibez):
		input = toFloat(str_input)		# oder toFloat(dummy)
		if input==None:
			QMessageBox.information(self.iface.mainWindow(),u"Hinweis", u"Fehlerhafter "+uibez)
			return None					
		return input					