#!/usr/bin/python
# -*- coding: utf-8 -*-

# Name           : MWLegende
# Description    : (c) Michael Wilhelm
# Date           : 25.08.2018
# Copyright      : (c) Michael Wilhelm
# Email          : mwilhelm@hs-harz.de
# Web            : http://www.miwilhelm.de



# Import the PyQt and QGIS libraries
from qgis.PyQt.QtCore import *  # QSettings, QTranslator, QCoreApplication
from qgis.PyQt.QtGui import *  # QIcon, QCursor
from qgis.PyQt.QtWidgets import *  # QAction

from qgis.gui import *

from qgis.core import * # QgsProject, QgsMapLayer, QgsWkbTypes, QgsMapLayerRegistry, QgsCoordinateTransform, QgsPoint, QgsRaster, QgsLayerTreeLayer, QgsLayerTreeGroup

# Initialize Qt resources from file resources.py
from .resources import *		# new  
from .basis import * 	# new
from .vectorlayer import * 	# new

import math
from .paletten import *

from  .uilegendrangecolor  import  uiLegendRangeColor
from  .uimenuefunction import uiMenuefunction

from .uiselectlayertype import uiSelectLayertype

class LegendRangeColor:

	def __init__(self, iface):
		# Save reference to the QGIS interface
		self.iface = iface
		self.canvas = self.iface.mapCanvas()
		self.view = iface.layerTreeView()

	def run(self):
		layers=[]
		
		
		layertype=""
		mainTree = self.iface.layerTreeView()
		currentgroup = mainTree.currentGroupNode()
		layer = getActiveShapeLayer(self.iface)
		if layer==None and currentgroup!=None:
			dialog=uiSelectLayertype()
			result = dialog.exec_()
			if not result:
				return		
				
			ergnr=-1
			if dialog.rbPointlayer.isChecked():
				layertype="point"
				layers = getPointLayerFromExistingGroup(self.iface,currentgroup)
			elif dialog.rbPolylinelayer.isChecked():
				layertype="line"
				layers = getPolylineLayerFromExistingGroup(self.iface,currentgroup)
			elif dialog.rbPolylgonlayer.isChecked():
				layertype="polygon"
				layers = getPolygonLayerFromExistingGroup(self.iface,currentgroup)
			else:
				QMessageBox.information(self.iface.mainWindow(),"Hinweis", u"Fehlerhafte Auswahl beim Ereignis")
				return
			# currentgroup
			if layers==None:
				QMessageBox.information(self.iface.mainWindow(),"Hinweis", "Die Gruppe hat keine Shapes des ausgewaehlten Typs")
				return
			if len(layers)==0:
				QMessageBox.information(self.iface.mainWindow(),"Hinweis", "Die Gruppe hat keine Shapes des ausgewaehlten Typs")
				return
				
		else:
			layer = getActiveShapeLayer(self.iface)
			if currentgroup==None and layer==None:
				QMessageBox.information(self.iface.mainWindow(),"Hinweis", "Es gab kein aktives Shape oder aktive Gruppe")
				return
			if layer.geometryType() == QgsWkbTypes.PointGeometry:
				layertype="point"
			elif layer.geometryType() == QgsWkbTypes.LineGeometry:
				layertype="line"
			elif layer.geometryType() == QgsWkbTypes.PolygonGeometry:
				layertype="polygon"
			else:
				QMessageBox.information(self.iface.mainWindow(),"Hinweis", u"Fehlerhafte Auswahl beim Bestimmen des Layertyps")
				return
			layers.append(layer)
			
		layer = layers[0]
		listekey=[]
		for feld in layer.fields():
			typename = feld.typeName().lower()
			#print(feld.typeName())
			# Date
			if typename=='real'  or typename=='double' or  typename=='int' or typename=='integer' or typename=='integer64' or typename=='integer32':
				listekey.append(feld.name())

				
		listekey.sort();
		dialog=uiMenuefunction()
		dialog.setValues(u"Klassifizierungsfeld",u"Wonach wird gruppiert?", listekey)
		result = dialog.exec_()
		if result:
			index = dialog.listWidget.currentRow()
			if index<0:
				return
		else:
			return
			
		searchfield=listekey[index]				
		#print(searchfield)
		
		
		layerListe=[]
		for layer in layers:
			for feat in layer.getFeatures():
				try:
					feld=feat[searchfield]
					layerListe.append(layer)
					break
				except:
					break
						
		if len(layerListe)==0:
			QMessageBox.information(self.iface.mainWindow(),u"Hinweis", u"Keine korrekten Layer trotz Gruppe vorhanden")
			return
			
			

		firstAlle=True
		minValueAlle=0
		maxValueAlle=0
		
		anzLayer=0
		listeAllerLayer=[]

		#legendIface = self.iface.legendInterface()
		for layer in layerListe:
			indexsearchfield=-1
			i=0
			for feld in layer.fields():
				print('searchfield',searchfield,'feld.name().lower()',feld.name())
				if searchfield == feld.name():
					indexsearchfield=i
					break
				else:
					i+=1
			print('         indexsearchfield',indexsearchfield)
			anzLayer+=1
			listeAllerLayer.append(layer.name())
			
			if indexsearchfield>=0:
				minValue = layer.minimumValue(indexsearchfield)
				maxValue = layer.maximumValue(indexsearchfield)
				print('indexsearchfield',indexsearchfield,'minValue',minValue,'maxValue',maxValue)
				if firstAlle:
					minValueAlle=minValue
					maxValueAlle=maxValue
					firstAlle=False
				else:
					if minValue<minValueAlle:
						minValueAlle=minValue
					if maxValue>maxValueAlle:
						maxValueAlle=maxValue			
			else:
				for feat in layer.getFeatures():
					contour=feat[searchfield]
					if contour!=None:
						if firstAlle:
							minValueAlle=contour
							maxValueAlle=contour
							firstAlle=False
						else:
							if contour<minValueAlle:
								minValueAlle=contour
							if contour>maxValueAlle:
								maxValueAlle=contour
						
						
		str_minValueAlle=u"{0:.2f}".format(float(minValueAlle))
		str_maxValueAlle=u"{0:.2f}".format(float(maxValueAlle))
		
		# dialog Variablen
		
		minValue = 0
		maxValue=0
		intervall=1
		strichdicke=0
		strichdicken1 = ["0.1","0.2","0.3","0.4","0.5","0.6","0.7","0.8","0.9",
            "1.0","1.2","1.3","1.4","1.5","1.6","1.7","1.8","1.9",
            "2.0","2.2","2.3","2.4","2.5","2.6","2.7","2.8","2.9",
            "3.0","3.2","3.3","3.4","3.5","3.6","3.7","3.8","3.9","4.0"]
		
		anzIntervalle1 = ["2","3","4","5","6","7","8","9","10","11","12","13","14","15","16","17","18","19","20"]
		farbpalette = ['Niederschlag (12 Farben)','LandCover (12 Farben)',\
		    'Gelb,Grün,Dunkelblau Farbverlauf',  'Grün,brauner Farbverlauf',\
		    'blauer Farbverlauf', 'roter Farbverlauf','brauner Farbverlauf','grüner Farbverlauf','grauer Farbverlauf',\
		     'alles hellrot', 'alles rot','alles hellblau', 'alles blau',u'alles hellgrün',\
	         u'alles grün','alles helllila', 'alles lila','alles hellorange', 'alles orange'	 ]
		intervall=1
		while True:
			dialog = uiLegendRangeColor()
			dialog.setValues(listeAllerLayer, minValueAlle,maxValueAlle)
			
			dialog.tMin.setText(str_minValueAlle)
			dialog.tMinVorgabe.setText(str_minValueAlle)
			dialog.tMax.setText(str_maxValueAlle)
			dialog.tMaxVorgabe.setText(str_maxValueAlle)
				
			if layertype=="point":
				dialog.labelStrichdickeMin.setText(u"Min. Markergröße")
				dialog.labelStrichdickeMax.setText(u"Max. Markergröße")
			elif layertype=="line":
				dialog.labelStrichdickeMin.setText(u"Min. Linienstärke")
				dialog.labelStrichdickeMax.setText(u"Max. Linienstärke")				
			elif layertype=="polygon":
				dialog.labelStrichdickeMin.setVisible(False)
				dialog.labelStrichdickeMax.setVisible(False)
				dialog.cbMinStrichdicke.setVisible(False)
				dialog.cbMaxStrichdicke.setVisible(False)
			
			# Layer Liste
			model = QStandardItemModel(dialog.listView)
			for layer in layerListe:
				item = QStandardItem(layer.name())
				# item.setCheckable(True)		# Add a checkbox to it
				model.appendRow(item)			# Add the item to the model
			dialog.listView.setModel(model)		
			
			
			result = dialog.exec_()
			if result:			
				
				minValue = self.checkInput(dialog.tMin.text(),"Minimaler Kategoriewert")
				if minValue==None:
					continue					
				maxValue = self.checkInput(dialog.tMax.text(),"Maximaler Kategoriewert")
				if maxValue==None:
					continue
				
				if minValue>=maxValue:
					QMessageBox.information(self.iface.mainWindow(),u"Hinweis", u"Der Anfangswert muss kleiner als der Endwert sein")
					continue

				
				anzIntervalle=-1
				if dialog.rbIntervall.isChecked():
					intervall = None
					if dialog.rb01.isChecked():
						intervall=0.1
					elif dialog.rb05.isChecked():
						intervall=0.5
					elif dialog.rb1.isChecked():
						intervall=1
					elif dialog.rb2.isChecked():
						intervall=2
					elif dialog.rb3.isChecked():
						intervall=3
					elif dialog.rb4.isChecked():
						intervall=4
					elif dialog.rb5.isChecked():
						intervall=5
					elif dialog.rb10.isChecked():
						intervall=10
						
					elif dialog.rb20.isChecked():
						intervall=20
					elif dialog.rb50.isChecked():
						intervall=50
					elif dialog.rb100.isChecked():
						intervall=100
					elif dialog.rb250.isChecked():
						intervall=250
					elif dialog.rb500.isChecked():
						intervall=500
					elif dialog.rb1000.isChecked():
						intervall=1000
					elif dialog.rb2500.isChecked():
						intervall=2500
					elif dialog.rb5000.isChecked():
						intervall=5000
						
						
					if intervall==None:
						QMessageBox.information(self.iface.mainWindow(),u"Hinweis", u"Der Intervallwert konnte nicht ermittelt werden")
						continue
						

					if intervall>maxValue-minValue:
						QMessageBox.information(self.iface.mainWindow(),u"Hinweis", u"Der Intervallwert ist groesser als die Differenz")
						continue
						
					hoehe = minValue
					anzIntervalle=0
					while hoehe<=maxValue+intervall*0.1:
						anzIntervalle+=1
						hoehe+=intervall						
					if anzIntervalle>20:
						anzIntervalle=20
					
				else:
					index = dialog.cbAnzIntervalle.currentIndex()
					if index<0:
						continue
					anzIntervalle=index+2
					intervall = (maxValue-minValue)/anzIntervalle
					#print('dialog.cbAnzIntervalle.currentIndex()',index,'maxValue-minValue',maxValue-minValue,'intervall',intervall)

					
				palettenIndex =  dialog.cbFarbpalette.currentIndex()
				str_strichdicke=dialog.cbMinStrichdicke.currentText()
				minStrichdicke = self.checkInputPositiv(str_strichdicke,"Min. Strichdicke")
				if minStrichdicke==None:
					continue
				
				str_strichdicke=dialog.cbMaxStrichdicke.currentText()
				maxStrichdicke = self.checkInputPositiv(str_strichdicke,"Max. Strichdicke")
				if maxStrichdicke==None:
					continue
				#if palettenIndex==0:
				
				
				break
				
			else:
				return	# Abbruch

		palette=[]
		if palettenIndex==0:
			palette = createPaletteNiederschlag(anzIntervalle)
		elif palettenIndex==1:
			palette = createPaletteLandCover(anzIntervalle)
			
		elif palettenIndex==2:
			palette = createPaletteGelbGruenDunkelblau(anzIntervalle)
			
			
		elif palettenIndex==3:
			palette = createPaletteGruenBrauner(anzIntervalle)

			
		elif palettenIndex==4:
			palette = createPaletteBlauerFarbverlauf(anzIntervalle)
		elif palettenIndex==5:
			palette = createPaletteRoterFarbverlauf(anzIntervalle)
		elif palettenIndex==6:
			palette = createPaletteBraunerFarbverlauf(anzIntervalle)
		elif palettenIndex==7:
			palette = createPaletteGruenerFarbverlauf(anzIntervalle)
		elif palettenIndex==8:
			palette = createPaletteGray(anzIntervalle)

			
		elif palettenIndex==9:
			palette = createHellrot(anzIntervalle)
		elif palettenIndex==10:
			palette = createRot(anzIntervalle)
		elif palettenIndex==11:
			palette = createHellblau(anzIntervalle)
		elif palettenIndex==12:
			palette = createBlau(anzIntervalle)
		elif palettenIndex==13:
			palette = createHellgruen(anzIntervalle)
		elif palettenIndex==14:
			palette = createGruen(anzIntervalle)
		elif palettenIndex==15:
			palette = createHelllila(anzIntervalle)
		elif palettenIndex==16:
			palette = createLila(anzIntervalle)
		elif palettenIndex==17:
			palette = createHellorange(anzIntervalle)
		elif palettenIndex==18:
			palette = createOrange(anzIntervalle)
			
		else:
			palette = createPaletteGray(anzIntervalle)
			QMessageBox.information(self.iface.mainWindow(),u"Hinweis", u"Keine korrekten Wert für die Palette: "+palettenIndex)
			

		
		if dialog.chkUmdrehen.isChecked():
			palette.reverse()
		
		if dialog.chkLog.isChecked():
			log=True
		else:
			log=False
			
		
		#print("minH",minH)
		#print("maxH",maxH)
		#print("intervall",intervall)
		#print("strichdicke",strichdicke)
		
		
		
		minvalues=[]
		maxvalues=[]
		labels=[]
		if log and minValue>0:
			diff=math.log10(maxValue)-math.log10(minValue)
			#print("anzIntervalle",anzIntervalle)
			difflog = diff/anzIntervalle
			#print("difflog",difflog)
			valuelog = math.log10(minValue)
			#print("valuelog",valuelog)
			maxlog = math.log10(maxValue)
			
			for i in range(0,anzIntervalle):
				v1=math.pow(10, valuelog)
				v2=math.pow(10, valuelog+difflog)
				#print("   valuelog",valuelog,v1,v2)
				minvalues.append(v1)
				maxvalues.append(v2)
				str1=u"{0:.2f}".format(float(v1))
				str2=u"{0:.2f}".format(float(v2))
				labels.append(str1+"  "+str2)
				valuelog+=difflog
		else:
			value = minValue
			for i in range(0,anzIntervalle):
				#print("value",value)
								
				
				if i==0:
					minvalues.append(value-1)  # echte Werte 
					str1=u"{0:.2f}".format(float(value))
				else:
					minvalues.append(value)
					str1=u"{0:.2f}".format(float(value))
				
				if i<(anzIntervalle-1):
					maxvalues.append(value+intervall)
					str2=u"{0:.2f}".format(float(value+intervall))
				else:
					maxvalues.append(value+intervall+1)
					str2=u"{0:.2f}".format(float(value+intervall))  # Rundungsfehler
				labels.append(str1+"  "+str2)
				value+=intervall
		
		if maxStrichdicke==minStrichdicke:
			diffStrichdicke=0
			strichdicke=minStrichdicke
		else:
			diffStrichdicke = (maxStrichdicke-minStrichdicke)/len(labels)
			if diffStrichdicke==0:
				diffStrichdicke=0.5
			strichdicke = minStrichdicke
			
			
		#print('anz paletten',len(palette),'len(labels)',len(labels))
		
		markersymbol = "circle"
		
		range_list = []
		for i in range(0,len(labels)):
			if layertype=="point":
				#symb = QgsSymbol.defaultSymbol( QgsWkbTypes.PointGeometry )
				symb = QgsMarkerSymbol.createSimple({'name' : markersymbol, 'color' : 'red', 'style':'no','width_border':'0.0', 'outline_color': 'black'})
			elif layertype=="line":
				symb = QgsSymbol.defaultSymbol( QgsWkbTypes.LineGeometry )
			elif layertype=="polygon":
				symb = QgsSymbol.defaultSymbol( QgsWkbTypes.PolygonGeometry )
			
			# symbols = layer.renderer().symbols()
			colorindex = i % len(palette)
			#print(i,colorindex,len(palette))
			color = QColor(palette[colorindex].r, palette[colorindex].g, palette[colorindex].b)
			symb.setColor( color )
			symb.setOpacity(1)
			symb.setOpacity(1)
			#symb.symbolLayer(0).setBorderStyle( Qt.NoPen )	# Umrandung bei Flächen
			
			if layertype=="point":
				symb.setSize(strichdicke) 
				symbolLayer = symb.symbolLayer(0)
				symbolLayer.setStrokeColor( color )
			elif layertype=="line":
				symb.setWidth(strichdicke) 
			elif layertype=="polygon":
				pass
			
			label = labels[i]			# +": "+str(minvalues[i])+" -> "+str(maxvalues[i])
			range_renderer = QgsRendererRange(minvalues[i], maxvalues[i], symb, label)
			
			range_list.append(range_renderer)
			
			strichdicke+=diffStrichdicke

		#QMessageBox.information(self.iface.mainWindow(),u"Hinweis", u"Nach loop (3)")	
		#renderer = QgsGraduatedSymbolRenderer('', range_list)
				# renderer.setMode(QgsGraduatedSymbolRenderer.EqualInterval)
		#renderer.setClassAttribute(searchfield)
		
		for layer in layerListe:
			renderer = QgsGraduatedSymbolRenderer('', range_list)
			renderer.setClassAttribute(searchfield)
			layer.setRenderer(renderer)
			layer.setOpacity(1)
			self.iface.layerTreeView().refreshLayerSymbology(layer.id())
				
		self.iface.mapCanvas().refresh()
		for layer in layerListe:
			layer.triggerRepaint()  	# repaintRequested

			
			
	def checkInputPositiv(self,str_input,uibez):
		input = toFloat(str_input)		# oder toFloat(dummy)
		if input==None:
			QMessageBox.information(self.iface.mainWindow(),u"Hinweis", u"Fehlerhafter "+uibez)
			return None					
		if input<0.0:
			QMessageBox.information(self.iface.mainWindow(),u"Hinweis", u"Fehlerhafter "+uibez+"\nsie muss >= 0 m sein")
			return None
		return input			
		
		
	def checkInput(self,str_input,uibez):
		input = toFloat(str_input)		# oder toFloat(dummy)
		if input==None:
			QMessageBox.information(self.iface.mainWindow(),u"Hinweis", u"Fehlerhafter "+uibez)
			return None					
		return input					