#!/usr/bin/python
# -*- coding: utf-8 -*-

# Name           : MWLegende
# Description    : (c) Michael Wilhelm
# Date           : 25.08.2018
# Copyright      : (c) Michael Wilhelm
# Email          : mwilhelm@hs-harz.de
# Web            : http://www.miwilhelm.de



from qgis.PyQt.QtCore import QSettings, QTranslator, QCoreApplication
from qgis.PyQt.QtGui import QIcon
from qgis.PyQt.QtWidgets import QAction, QMessageBox
from qgis.core import * # QgsProject, QgsMapLayer, QgsWkbTypes, QgsCoordinateTransform, QgsPoint, QgsRaster, QgsLayerTreeLayer, QgsLayerTreeGroup
from qgis.gui import *    # QgsMapTool  # neu

# QMessageBox

from .basis import * 	# new

# Initialize Qt resources from file resources.py
from .resources import *
import math


class MyQGisPoint():
	def __init__(self, x, y):
		self.x=x
		self.y=y
		self.diffPoint=0.0
		self.sumLaenge=0.0
		self.laenge=0.0  


class VectorLayer(QgsMapTool):

	# statische Variablen
	gewchange=False		# verhindert den Aufruf in IGMKataster
	gewedit=False		# zeigt an, dass das Gew. noch editiert wird.
						# Dient als Abfrage in "Neues Gewaesser"




def getActiveShapeLayer(iface):
	layer = iface.activeLayer()
	if(not layer):
		return None
		
	if layer.type() != QgsMapLayer.VectorLayer:
		return None
	return layer

	# Point, Line, Polygon, UnknownGeometry, NoGeometry
def getActivePointLayer(iface):
	layer = iface.activeLayer()
	if(not layer):
		return None

	if layer.type() != QgsMapLayer.VectorLayer:
		return None
	if layer.geometryType() != QgsWkbTypes.PointGeometry:
		return None
	return layer


def getActivePolylineLayer(iface):
	layer = iface.activeLayer()
	if(not layer):
		return None

	if layer.type() != QgsMapLayer.VectorLayer:
		return None
	if layer.geometryType() != QgsWkbTypes.LineGeometry:
		return None
	return layer


def getActivePolygonLayer(iface):
	layer = iface.activeLayer()
	if(not layer):
		return None

	if layer.type() != QgsMapLayer.VectorLayer:
		return None
	if layer.geometryType() != QgsWkbTypes.PolygonGeometry:
		return None
	return layer


# nur point and polyline
def getActiveFeature(layer, layerPoint):
	layer.removeSelection()
	
	shortestDistance = float("inf")
	closestFeatureId = -1
	activeFeature=None
	type=layer.geometryType()
	if type==QgsWkbTypes.PolygonGeometry:
		for feat in layer.getFeatures():
			if feat!=None and feat.geometry()!=None and  feat.geometry().contains( QgsGeometry.fromPointXY( layerPoint) ):
				closestFeatureId = feat.id()
				activeFeature=feat
				# print("gefunden")
				return closestFeatureId, activeFeature
		# print("keine Daten gefunden}")
		# Looks like the vector layers were found - do nothing
		return None,None
	else:
		# Loop through all features in the layer
		if layer.featureCount==0:
			return None,None
		elif layer.featureCount==1:
			for feat in layer.getFeatures():
				return feat.id(), feat
		else:
			for feat in layer.getFeatures():  # QgsFeatureIterator
				if feat!=None and feat.geometry()!=None :
					dist = feat.geometry().distance( QgsGeometry.fromPointXY( layerPoint) )  # new
					if dist < shortestDistance:
						shortestDistance = dist
						closestFeatureId = feat.id()
						activeFeature=feat
			# print("keine Daten gefunden}")
			# Looks like the vector layers were found - do nothing
			if shortestDistance < 0:
				return None,None

	return closestFeatureId, activeFeature


def getSelectedFeatures(layer):
	return layer.selectedFeatures()
	# return layer.selectedFeaturesIterator()

	
def getSelectedFeatures_old(layer):
	features = []
	for featid in layer.selectedFeaturesIds():
		for feat in layer.getFeatures():
			if featid==feat.id():
				features.append(feat)
	return features
	

def getRealCoordinate(iface, layer,x,y):
	layerPoint = iface.mapCanvas().getCoordinateTransform().toMapCoordinates(x, y)
	return layerPoint


def distance(p1, p2):
	dx = p1.x - p2.x
	dy = p1.y - p2.y
	return math.sqrt(dx * dx + dy * dy)

def calcLastPoint( p1,p2, difflaenge, mittig):
	pEnde=None
	x1 = p1.x()
	x2 = p2.x()
	y1 = p1.y()
	y2 = p2.y()
	dx = x1 - x2
	dy = y1 - y2
	if mittig:
		x3=x1
		y3=y1
	else:
		x3=x2
		y3=y2
	# print("dx",dx,"dy",dy)
	# print("difflaenge",difflaenge)
	if abs(dx)<1.0E-5:
		#	senkrecht
		# print("senkrecht")
		if y1<y2:
			# von p1(unten) nach p2(oben)
			pEnde = QgsPointXY(x2, y3+difflaenge)
		else:
			# von p1(oben) nach p2(unten)
			pEnde = QgsPointXY(x1, y3-difflaenge)
	elif abs(dy)<1.0E-5:
		#	waagerecht
		# print("waagerecht")
		if x1<x2:
			# von p1(links) nach p2(rechts)
			pEnde = QgsPointXY(x3+difflaenge, y2)
		else:
			# von p2(links) nach p1(rechts)
			pEnde = QgsPointXY(x3-difflaenge, y1)
	else:
		# print("schief")
		m = dy/dx
		# print("m",m)
		alpharad = math.atan(m)
		# restliche Hypothenuse			
		# nun die echten lokalen Koordinaten !
		xneu = difflaenge*math.sin(alpharad)
		yneu = difflaenge*math.cos(alpharad)
		# print("xneu",xneu,"yneu",yneu)
		if m>0:
			if x2<x1:		# Linie /
				x = x3-yneu
				y = y3-xneu
			else:
				x = x3+yneu
				y = y3+xneu
		else:
			if x1<x2:		# Linie \
				x = x3+yneu		#	+
				y = y3+xneu		#	-
				# print("3")
			else:
				x = x3-yneu		#	-
				y = y3-xneu		#	+
				# print("4")
		pEnde = QgsPointXY(x, y)
		
	return pEnde
	

def changeLinie(iface, geom, neuelaenge):
	geoXY = getGeoXYSinglePolyLine(geom) #geom.asPolyline()
	aktLaenge = geom.length()
	nLen = len(geoXY)
	if nLen<=1:
		QMessageBox.information(iface.mainWindow(),u"Hinweis", u"Das Gewaesser hat keine zwei Stuetzpunkte")
		return None
	# print("aktLaenge",aktLaenge)
	# print("neuelaenge",neuelaenge)
	# print("len(geoXY)",nLen)	# 9
	if neuelaenge<=aktLaenge:
		# print("neuelaenge<aktlaenge")
		gesamtlaenge = 0
		ergpoints = []
		ergpoints.append(geoXY[0])
		index = -1
		for i in range(0,nLen-1):	# range(0,10)  geht 0 bis 9 !!!
			p1 = geoXY[i]
			p2 = geoXY[i+1]
			dx = p1.x() - p2.x()
			dy = p1.y() - p2.y()
			laenge = math.sqrt(dx * dx + dy * dy)
			
			# print("gesamtlaenge",gesamtlaenge)
			if (gesamtlaenge+laenge)<=neuelaenge:
				ergpoints.append(p2)
				gesamtlaenge+=laenge
			else:
				index=i
				break
		if gesamtlaenge<neuelaenge:
			# print("index",index)
			if index<len(geoXY)-1:
				# abschnitt noch vorhanden
				p1 = geoXY[index]
				p2 = geoXY[index+1]
				difflaenge = neuelaenge-gesamtlaenge
				pEnde = calcLastPoint(p1,p2, difflaenge, True)
				ergpoints.append(pEnde)
			else:
				QMessageBox.information(iface.mainWindow(),u"Hinweis", u"Die neue Gewaesserlaenge konnte nicht berechnet werden (Fehler changeLinie")
				return None
		return ergpoints
	else:
		# bis>aktLaenge:  nun verlaengern			
		gesamtlaenge = aktLaenge
		# alle Punkte in das Feld
		ergpoints = []
		for i in range(0,nLen):	# range(0,10)  geht 0 bis 9 !!!
			ergpoints.append(geoXY[i])
			
		p1 = geoXY[nLen-2]
		p2 = geoXY[nLen-1]
		difflaenge = neuelaenge-gesamtlaenge
		pEnde = calcLastPoint(p1,p2, difflaenge, False)
		ergpoints.append(pEnde)
		return ergpoints


# qgsPoints  sind die Koordinaten des Gewaessers
def calcStation(iface, qgsPoints, shpPoint_x, shpPoint_y):
	shpPoint = MyQGisPoint(shpPoint_x, shpPoint_y)
	gewpoints=[]
	xold=None
	yold=None
	i=-1
	for qpoint in qgsPoints:
		i+=1
		# print(i,qpoint.x(),qpoint.y())
		if xold==None:
			p=MyQGisPoint(qpoint.x(),qpoint.y())
			gewpoints.append(p)
		else:
			if xold!=qpoint.x() or yold!=qpoint.y() :
				p=MyQGisPoint(qpoint.x(),qpoint.y())
				gewpoints.append(p)
		xold = qpoint.x()
		yold = qpoint.y()
	if (len(gewpoints)<2):
		return -1
		
	#print("len(qgsPoints)",len(qgsPoints))
	#print("len(gewpoints)",len(gewpoints))
	if (len(gewpoints)==2):
		laenge=0.0
		p1 = gewpoints[0]
		p2 = gewpoints[1]
		p1.laenge=distance(p1,p2)
		p = verticalPoint(p1, p2, shpPoint)			
		station = p1.sumLaenge + distance(p1, p)
		#print("shpPoint_x",shpPoint_x)
		#print("shpPoint_y",shpPoint_y)
		#print("station",station)
	
	else:
		#print("shpPoint_x",shpPoint_x)
		#print("shpPoint_y",shpPoint_y)
		laenge=0.0
		for i in range(0,len(gewpoints)-1):  # 0 until n-1
			p1 = gewpoints[i]
			p2 = gewpoints[i+1]
			p1.laenge=distance(p1,p2)
			laenge+=p1.laenge

		#print("laenge",laenge)
		minDistanz = laenge*100;
		#print("minDistanz",minDistanz)
		minIndex = -1
		for i in range(0,len(gewpoints)):  # 0 until n-1
			p1 = gewpoints[i]
			p1.diffPoint = distance(p1, shpPoint)
			#print("i",i,"x",p1.x,"y",p1.y,"minDistanz",p1.diffPoint)
			if p1.diffPoint < minDistanz:
				minDistanz = p1.diffPoint;
				minIndex = i;
			if i > 0:
				p2 = gewpoints[i-1]
				p1.sumLaenge = p2.sumLaenge + p2.laenge


		#print("minIndex",minIndex)
		station=-1.0;
		if minIndex==-1:
			# print("Keine Loesung gefunden")
			return None
		elif minIndex==0:
			p1 = gewpoints[0]
			p2 = gewpoints[1]
			p = verticalPoint(p1, p2, shpPoint)
			station = distance(p1, p)
		elif minIndex==len(gewpoints)-1:
			#print("ende linie")
			p1 = gewpoints[len(gewpoints)-2]
			p2 = gewpoints[len(gewpoints)-1]
			p = verticalPoint(p1, p2, shpPoint)
			station = p1.sumLaenge + distance(p1, p)
		else:
			p1 = gewpoints[minIndex-1]
			p2 = gewpoints[minIndex]
			p3 = gewpoints[minIndex+1]
			pPrev = verticalPoint(p1, p2, shpPoint)
			pNext = verticalPoint(p2, p3, shpPoint)
			# check, ob die Punkte im Spannviereck liegen  x1/y1  x2/y2
			#  Prev   P1/p2
			#  Next   P2/p3
			prevViereck=False
			nextViereck=False
			b1=min(p1.x,p2.x)<=shpPoint.x
			b2=shpPoint.x<=max(p1.x,p2.x)
			b3=min(p1.y,p2.y)<=shpPoint.y
			b4=shpPoint.y<=max(p1.y,p2.y)
			
			#print("p1.x",str(p1.x), "p1.y",str(p1.y))
			#print("p2.x",str(p2.x), "p2.y",str(p2.y))
			#print("p3.x",str(p3.x), "p3.y",str(p3.y))
			
			if p1.x==p2.x and b3 and b4:
				prevViereck=True  # senkrecht
			elif p1.y==p2.y and b1 and b2:
				prevViereck=True  # waagerecht
			elif abs(p1.x-p2.x)<10 and b3 and b4:
				prevViereck=True  # fast senkrecht
			elif abs(p1.y-p2.y)<10 and b1 and b2:
				prevViereck=True  # fast waagerecht
			elif b1 and b2 and b3 and b4:
				prevViereck=True

			b1=min(p2.x,p3.x)<=shpPoint.x
			b2=shpPoint.x<=max(p2.x,p3.x)
			b3=min(p2.y,p3.y)<=shpPoint.y
			b4=shpPoint.y<=max(p2.y,p3.y)
			# print(b1,b2,b3,b4)
			if p2.x==p3.x and b3 and b4:
				nextViereck=True  # senkrecht
			elif p2.y==p3.y and b1 and b2:
				nextViereck=True  # waagerecht
			elif abs(p2.x-p3.x)<10 and b3 and b4:
				nextViereck=True  # fast senkrecht
			elif abs(p2.y-p3.y)<10 and b1 and b2:
				nextViereck=True  # fast waagerecht
			elif b1 and b2 and b3 and b4:
				nextViereck=True
				
				
			#print("prevViereck",prevViereck)
			#print("nextViereck",nextViereck)
			stationPrev = p1.sumLaenge+distance(p1, pPrev)
			stationNext = p2.sumLaenge+distance(p2, pNext)
			diffPrev = distance(shpPoint, pPrev)
			diffNext = distance(shpPoint, pNext)
			#print("diffprev",diffPrev,"stationPrev",stationPrev)
			#print("diffNext",diffNext,"stationNext",stationNext)
			if prevViereck and nextViereck:
				if diffPrev < diffNext:
					station=stationPrev
				else:
					station=stationNext
			else:
				if prevViereck:
					station=stationPrev
				else:
					if nextViereck:
						station=stationNext
					else:
						QMessageBox.information(iface.mainWindow(),"Hinweis", "Es konnte keine Station am Mausklick gefunden werden")
						station=None
	return station

def verticalPoint(p1, p2, p3):
	p = MyQGisPoint(0.0, 0.0)
	if p1.y==p2.y:
		# waagerecht
		p.x = p3.x
		p.y = p1.y
	elif p1.x==p2.x:
		# senkrecht
		p.y = p3.y
		p.x = p1.x
	else:
		m1 = (p1.y - p2.y) / (p1.x - p2.x)
		b1 = p1.y - m1 * p1.x
		m3 = -1.0 / m1
		b3 = p3.y - m3 * p3.x
		x3 = (b3 - b1) / (m1 - m3)
		y3 = m3 * x3 + b3
		p.x = x3
		p.y = y3

	return p;
	
def qgsdistance(p1, p2):
	dx = p1.x() - p2.x()
	dy = p1.y() - p2.y()
	return math.sqrt(dx * dx + dy * dy)


def createPolyline(iface,feat, von,bis, newfeat):
	if von==bis:
		return None

	geom = feat.geometry()
	laenge = geom.length()
	#qgsPoints = geom.asPolyline()
	qgsPoints = getGeoXYSinglePolyLine(geom)  # basis 
	if len(qgsPoints)==0:
		QMessageBox.information(iface.mainWindow(),u"Hinweis", u"Die Linie hat keine Punkte")
		return None
		
	gewpoints=[]
	xold=None
	yold=None
	for qpoint in qgsPoints:
		# print("   x: ",qpoint.x(), "   y: ",qpoint.y())
		if xold==None:
			# p=MyQGisPoint(qpoint.x(),qpoint.y())
			gewpoints.append(qpoint)
		else:
			if xold!=qpoint.x() or yold!=qpoint.y() :
				# p=MyQGisPoint(qpoint.x(),qpoint.y())
				gewpoints.append(qpoint)
		xold = qpoint.x()
		yold = qpoint.y()
	if (len(gewpoints)<2):
		return -1
		
	
	ergpoints=[]
	if von > laenge:
		von = laenge
	if bis > laenge:
		bis = laenge
		
	if laenge>0:
		if von==0.0:
			ergpoints.append(gewpoints[0])
		else:
			geom2=geom.interpolate(von)
			ergpoints.append(geom2.asPoint())

		currentdistance = 0
		for i in range(0,len(gewpoints)-2):
			p1 = gewpoints[i]
			p2 = gewpoints[i+1]
			# print("p1",p1.x(),p1.y())
			# print("p2",p2.x(),p2.y())
			currentdistance+=qgsdistance(p1,p2)
			if currentdistance>von and currentdistance<bis:
				ergpoints.append(p2)

		geom2=geom.interpolate(bis)
		p = geom2.asPoint()
		ergpoints.append(p)
	#print("createPolyline von:",von,"ergpoints[0]",ergpoints[0])
	shpLine=QgsGeometry.fromPolylineXY(ergpoints)
	newfeat.setGeometry(shpLine)
	return 0
	
	
	
def createPoint(iface, feat, von,bis, newfeat):
	if von==bis:
		geom = feat.geometry()
		laenge = geom.length()
		#qgsPoints = geom.asPolyline()
		qgsPoints = getGeoXYSinglePolyLine(geom)
		if len(qgsPoints)==0:
			QMessageBox.information(iface.mainWindow(),u"Hinweis", u"Die Linie hat keine Punkte")
			return None
			
		if von > laenge:
			von = laenge
		
		if laenge>0:
		
			# qgsPoints = geom.asPolyline()
			qgsPoints = getGeoXYSinglePolyLine(geom)  # basis
			if len(qgsPoints)==0:
				# multipolyline
				multilines = geom.asMultiPolyline()
				#print("length(multilines) ", len(multilines) )
				#print("gesamtlaenge: ", laenge )					
				loopvon=von
				for line in multilines:
					# line ist eine Liste      edef QVector< QgsPoint > 	QgsPolyline
					anzkoords=len(line)   # anzahl koordinaten
					shpLine=QgsGeometry.fromPolylineXY(line)
					#print("shpLine: ", shpLine.length() )
					polylen=shpLine.length()
					if (loopvon<=polylen):
						geom2=shpLine.interpolate(loopvon)
						p = geom2.asPoint()
						shpPoint=QgsGeometry.fromPointXY(p)
						newfeat.setGeometry(shpPoint)
						#print("loopvon",loopvon)
						return shpPoint
					else:
						loopvon-=polylen 
					
			else:
				# einfache Linie				
				geom2=geom.interpolate(von)
				p = geom2.asPoint()
				shpPoint=QgsGeometry.fromPointXY(p)
				newfeat.setGeometry(shpPoint)
				return shpPoint
		else:
			QMessageBox.information(self.iface.mainWindow(),"Hinweis", u"Die Laenge der Linie ist null, ABBRUCH")
			return None
	else:
		QMessageBox.information(self.iface.mainWindow(),"Hinweis", u"Die Stationen von und bis sind nicht gleich, ABBRUCH")
		return None
		
		
				

def setColorWidth(layer, red,green,blue):
	symbols = layer.rendererV2().symbols()  # QgsMarkerSymbolV2
	# void 	setAngle (double angle)
	sym=symbols[0]
	sym.setOpacity(1)
	sym.setColor( QColor.fromRgb(red,green,blue) )   # rot
	layer.triggerRepaint()  # repaintRequested()
			
			
			
def setColorWidth(layer, red,green,blue, width):
	symbols = layer.rendererV2().symbols()  # QgsMarkerSymbolV2
	# void 	setAngle (double angle)
	sym=symbols[0]
	sym.setOpacity(1)
	sym.setWidth(width)  # QgsLineSymbolV2
	sym.setColor( QColor.fromRgb(red,green,blue) )   # rot
	layer.triggerRepaint()  # repaintRequested()
			
			